__all__ = [
    'apps',
    'template',
    'config',
    'completion',
    'feedback',
    'login',
    'logs',
    'maintenance',
    'releases',
    'status',
    'test',
    'update',
    'deploy',
    'logout'
]
